#include "PainterEngine_Application.h"
PX_Application App;



px_void PX_Application_RenderDisplaySurface(PX_Application* pApp)
{
	px_float renderx, rendery;
	px_float hz;
	px_float center_view_hz=pApp->center_view_hz;
	px_int lindex,rindex,index;
	PX_SurfaceClearAll(&pApp->surface_freqdisplay, PX_COLOR(255, 255, 255, 255));
	
	lindex = (px_int)(pApp->view_index- pApp->surface_freqdisplay.width/2.f/pApp->pixels_per_step);
	rindex = (px_int)(pApp->view_index + pApp->surface_freqdisplay.width /2.f/ pApp->pixels_per_step);


	if (lindex<0)
	{
		lindex = 0;
	}
	
	if (lindex >= pApp->samples_count)
	{
		lindex = pApp->samples_count - 1;
	}

	if (rindex < 0)
	{
		rindex = 0;
	}

	if (rindex >= pApp->samples_count)
	{
		rindex = pApp->samples_count - 1;
	}

	if (lindex<0)
	{
		return;
	}

	if(center_view_hz< pApp->surface_freqdisplay.height / 2.f / pApp->pixels_per_hz)
		center_view_hz = pApp->surface_freqdisplay.height / 2.f / pApp->pixels_per_hz;

	for (index= lindex;index<=rindex;index++)
	{
		renderx = pApp->surface_freqdisplay.width / 2.f+(index - pApp->view_index) * pApp->pixels_per_step;
		hz = pApp->pitch[index];
		rendery = pApp->surface_freqdisplay.height-(pApp->surface_freqdisplay.height / 2.f + (hz - center_view_hz) * pApp->pixels_per_hz);
		if(hz!=0)
		PX_GeoDrawSpray(&pApp->surface_freqdisplay, renderx, rendery, 3, PX_COLOR(255, 0, 64, 255));
	}

	PX_GeoDrawLine(&pApp->surface_freqdisplay, pApp->surface_freqdisplay.width / 2, 0, pApp->surface_freqdisplay.width / 2, pApp->surface_freqdisplay.height, 1, PX_COLOR_RED);

	do 
	{
		//render standard frequencies
		px_int i;
		px_float freq[51];
		for (i=36;i<36+51;i++)
		{
			freq[i-36]=(px_float)((440.0 / 32.0)* PX_pow(2, (i - 9.0) / 12.0));
		}
		for (i=0;i<51;i++)
		{
			px_char content[64];
			rendery = pApp->surface_freqdisplay.height - (pApp->surface_freqdisplay.height / 2.f + (freq[i] - center_view_hz) * pApp->pixels_per_hz);
			if (rendery>0&&rendery<pApp->surface_freqdisplay.height)
			{
				PX_GeoDrawLine(&pApp->surface_freqdisplay, 0, (px_int)rendery, pApp->surface_freqdisplay.width, (px_int)rendery, 1, PX_COLOR_RED);
				PX_sprintf1(content, sizeof(content), "%1.2 Hz", PX_STRINGFORMAT_FLOAT((freq[i])));
				PX_FontModuleDrawText(&pApp->surface_freqdisplay, 0, 1, (px_int)rendery - 1, PX_ALIGN_LEFTBOTTOM, content, PX_COLOR_BLUE);
			}
		}

	} while (0);

	do
	{
		//render view frequencies
		px_char content[64];
		px_float value;
		value=(px_float)(800-PX_Object_SliderBarGetValue(pApp->sliderbar_hzrange));
		rendery = pApp->surface_freqdisplay.height - (pApp->surface_freqdisplay.height / 2.f + (value - center_view_hz) * pApp->pixels_per_hz);
		PX_GeoDrawLine(&pApp->surface_freqdisplay, 0, (px_int)rendery, pApp->surface_freqdisplay.width, (px_int)rendery, 1, PX_COLOR_GREEN);
		PX_sprintf1(content, sizeof(content), "%1.2 Hz", PX_STRINGFORMAT_FLOAT((value)));
		PX_FontModuleDrawText(&pApp->surface_freqdisplay, 0, pApp->surface_freqdisplay.width - 1, (px_int)rendery - 1, PX_ALIGN_RIGHTBOTTOM, content, PX_COLOR_BLUE);
	} while (0);

	do 
	{
		px_char content[64];
		rendery = pApp->surface_freqdisplay.height - (pApp->surface_freqdisplay.height / 2.f + (pApp->pitch[pApp->view_index] - center_view_hz) * pApp->pixels_per_hz);
		PX_GeoDrawCircle(&pApp->surface_freqdisplay, (px_int)pApp->surface_freqdisplay.width / 2, (px_int)rendery, 6, 1, PX_COLOR_RED);
		PX_sprintf1(content, sizeof(content), "%1.2 Hz", PX_STRINGFORMAT_FLOAT((pApp->pitch[pApp->view_index])));
		PX_FontModuleDrawText(&pApp->surface_freqdisplay, 0, pApp->surface_freqdisplay.width /2, (px_int)rendery - 8, PX_ALIGN_MIDBOTTOM, content, PX_COLOR_BLUE);
	} while (0);

}



px_void PX_Application_OnOpenFile(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Application* pApp = (PX_Application*)ptr;
#ifdef _MSC_VER
	do 
	{
		extern char* PX_OpenFileDialog(const char Filter[]);
		px_char *ppath;
		if (ppath=PX_OpenFileDialog(""))
		{
			if (ppath[0])
			{
				PX_SoundStaticDataFree(&pApp->sounddata);
				if (!PX_LoadSoundFromFile(&pApp->runtime.mp_resources, &pApp->sounddata, ppath))
				{
					return;
				}
			}
		}

	} while (0);
#else
	do 
	{
		extern px_void px_getfile();
		px_getfile();
	} while (0);
#endif // _MSC_VER

	pApp->progress_byte_offset = 0;
	pApp->samples_count = 0;
	pApp->state = PX_APPLICATION_STATE_PROCESSING;
	PX_Object_SliderBarSetValue(pApp->sliderbar_progress, 0);
}

px_void PX_Application_OnProgressSliderChanged(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	px_float value = PX_Object_SliderBarGetValue(pObject)/10000.0f;
	PX_Application* pApp = (PX_Application*)ptr;
	pApp->state = PX_APPLICATION_STATE_SHOW;
	PX_SoundPlayClear(&pApp->runtime.soundplay);
	pApp->view_index = ((px_int)(value * pApp->samples_count));

}

px_void PX_Application_OnHzSliderChanged(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	px_float value = PX_Object_SliderBarGetValue(pObject)*1.0f;
	PX_Application* pApp = (PX_Application*)ptr;
	pApp->center_view_hz = 800-value;
}

px_void PX_Application_OnPlay(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	px_float value;
	PX_Application* pApp = (PX_Application*)ptr;
	if (pApp->sounddata.size==0)
	{
		return;
	}
	value = PX_Object_SliderBarGetValue(pApp->sliderbar_progress) / 10000.f;
	PX_SoundPlayClear(&pApp->runtime.soundplay);
	PX_SoundPlayAdd(&pApp->runtime.soundplay, PX_SoundCreate(&pApp->sounddata, PX_FALSE));
	pApp->runtime.soundplay.Sounds[0].offset = ((px_int)(value * pApp->sounddata.size)) / 4 * 4;
	pApp->state = PX_APPLICATION_STATE_PLAYING;
}

px_void PX_Application_OnStop(PX_Object* pObject, PX_Object_Event e, px_void* ptr)
{
	PX_Application* pApp = (PX_Application*)ptr;
	PX_SoundPlayClear(&pApp->runtime.soundplay);
	pApp->state = PX_APPLICATION_STATE_SHOW;
}

px_bool PX_ApplicationInitialize(PX_Application *pApp,px_int screen_width,px_int screen_height)
{
	PX_ApplicationInitializeDefault(&pApp->runtime, screen_width, screen_height);

	if (!PX_LoadTextureFromFile(&pApp->runtime.mp_resources,&pApp->logo,"assets/logo.png"))
	{
		return PX_FALSE;
	}
	if (!PX_SoundPlayInitialize(&pApp->runtime.mp_game,&pApp->runtime.soundplay))
	{
		return PX_FALSE;
	}

	if (!PX_AudioInitialize(&pApp->runtime.soundplay))
	{
		return PX_FALSE;
	}
	


	pApp->root = PX_ObjectCreate(&pApp->runtime.mp_ui, 0, 0, 0, 0, 0, 0, 0);
	pApp->image_logo = PX_Object_ImageCreate(&pApp->runtime.mp_ui, pApp->root, 0, 0, pApp->logo.width, pApp->logo.height, &pApp->logo);
	pApp->button_openfile = PX_Object_CursorButtonCreate(&pApp->runtime.mp_ui, pApp->root, 0, 0, 96, 32, "Open Wav", 0, PX_OBJECT_UI_DEFAULT_FONTCOLOR);
	pApp->button_play = PX_Object_CursorButtonCreate(&pApp->runtime.mp_ui, pApp->root, 0, 0, 96, 32, "Play", 0, PX_OBJECT_UI_DEFAULT_FONTCOLOR);
	pApp->button_stop = PX_Object_CursorButtonCreate(&pApp->runtime.mp_ui, pApp->root, 0, 0, 96, 32, "Stop", 0, PX_OBJECT_UI_DEFAULT_FONTCOLOR);

	pApp->sliderbar_progress = PX_Object_SliderBarCreate(&pApp->runtime.mp_ui, pApp->root, 0, 0, pApp->runtime.surface_width - 1, 24, PX_OBJECT_SLIDERBAR_TYPE_HORIZONTAL, PX_OBJECT_SLIDERBAR_STYLE_BOX);
	PX_Object_SliderBarSetRange(pApp->sliderbar_progress, 0, 10000);
	PX_ObjectRegisterEvent(pApp->sliderbar_progress, PX_OBJECT_EVENT_VALUECHANGED, PX_Application_OnProgressSliderChanged, pApp);

	pApp->sliderbar_hzrange = PX_Object_SliderBarCreate(&pApp->runtime.mp_ui, pApp->root, (px_int)pApp->runtime.surface_width-24, (px_int)pApp->image_logo->Height, 24, (px_int)(pApp->runtime.surface_height - 24-pApp->image_logo->Height), PX_OBJECT_SLIDERBAR_TYPE_VERTICAL, PX_OBJECT_SLIDERBAR_STYLE_BOX);
	
	PX_ObjectRegisterEvent(pApp->sliderbar_hzrange, PX_OBJECT_EVENT_VALUECHANGED, PX_Application_OnHzSliderChanged, pApp);
	

	pApp->button_openfile->x = pApp->image_logo->x + pApp->image_logo->Width+5;
	pApp->button_openfile->y = pApp->image_logo->y + pApp->image_logo->Height - pApp->button_openfile->Height;
	PX_ObjectRegisterEvent(PX_Object_CursorButtonGetPushButton( pApp->button_openfile), PX_OBJECT_EVENT_EXECUTE, PX_Application_OnOpenFile, pApp);

	
	pApp->button_play->x = pApp->button_openfile->x + pApp->button_openfile->Width + 5;
	pApp->button_play->y = pApp->button_openfile->y;
	PX_ObjectRegisterEvent(PX_Object_CursorButtonGetPushButton(pApp->button_play), PX_OBJECT_EVENT_EXECUTE, PX_Application_OnPlay, pApp);


	pApp->button_stop->x = pApp->button_openfile->x + pApp->button_openfile->Width + 5;
	pApp->button_stop->y = pApp->button_openfile->y;
	PX_ObjectRegisterEvent(PX_Object_CursorButtonGetPushButton(pApp->button_stop), PX_OBJECT_EVENT_EXECUTE, PX_Application_OnStop, pApp);

	pApp->pixels_per_step = 1.0f;
	pApp->pixels_per_hz = 1.0f;

	PX_Object_SliderBarSetRange(pApp->sliderbar_hzrange, 0,800 );
	PX_Object_SliderBarSetValue(pApp->sliderbar_hzrange, 800);

	pApp->messagebox = PX_Object_MessageBoxCreate(&pApp->runtime.mp_ui, pApp->root, 0);

	if (!PX_TextureCreate(&pApp->runtime.mp_ui, &pApp->surface_freqdisplay,(px_int)( pApp->runtime.surface_width-24), (px_int)(pApp->runtime.surface_height - pApp->image_logo->Height - 25)))
		return PX_FALSE;

	return PX_TRUE;
}
#define PX_APPLICATION_FRAME_SIZE 1024

px_void PX_ApplicationUpdate_ProcessingFrame(PX_Application* pApp)
{
	px_int readbytes;
	px_float frame[PX_APPLICATION_FRAME_SIZE] = {0};
	px_complex complex[PX_APPLICATION_FRAME_SIZE] = { 0 };
	if (readbytes = PX_SoundDataReadSamples(&pApp->sounddata, pApp->progress_byte_offset, PX_APPLICATION_FRAME_SIZE, frame, 0))
	{
		px_int i;
		for (i=0;i< PX_APPLICATION_FRAME_SIZE;i++)
		{
			complex[i].re = frame[i];
		}
		if(pApp->samples_count<PX_COUNTOF(pApp->pitch))
			pApp->pitch[pApp->samples_count]=PX_PitchEstimation(complex, PX_APPLICATION_FRAME_SIZE, 44100, 50, 800);

		pApp->samples_count++;
		pApp->progress_byte_offset += readbytes/4;//75% overlap

		do 
		{
			px_char content[64] = { 0 };
			PX_sprintf1(content, sizeof(content), "Processing:%1.2%", PX_STRINGFORMAT_FLOAT(pApp->progress_byte_offset * 100.f / pApp->sounddata.size));
			PX_Object_MessageBoxAlert(pApp->messagebox, content);
		} while (0);
	}
	else
	{
		pApp->state = PX_APPLICATION_STATE_SHOW;
		pApp->view_index = 0;
		
	}
}


px_void PX_ApplicationUpdate(PX_Application *pApp,px_dword elapsed)
{
	if (pApp->state==PX_APPLICATION_STATE_PROCESSING)
	{
		px_int i;
		for (i=0;i<32;i++)
		{
			PX_ApplicationUpdate_ProcessingFrame(pApp);
		}
		

		if (!pApp->messagebox->Visible)
		{
			PX_Object_MessageBoxAlert(pApp->messagebox,"processing");
		}
	}
	else if(pApp->state==PX_APPLICATION_STATE_SHOW)
	{
		if (pApp->messagebox->Visible)
		{
			PX_Object_MessageBoxClose(pApp->messagebox);
		}
		if (pApp->sounddata.size)
		{
			pApp->button_play->Visible = PX_TRUE;
		}
		else
		{
			pApp->button_play->Visible = PX_FALSE;
		}
		
		pApp->button_stop->Visible = PX_FALSE;
	}
	else if (pApp->state == PX_APPLICATION_STATE_PLAYING)
	{
		px_float value;
		if (pApp->messagebox->Visible)
		{
			PX_Object_MessageBoxClose(pApp->messagebox);
		}
		pApp->view_index=(px_int)((pApp->runtime.soundplay.Sounds[0].offset*1.0/ pApp->sounddata.size)*pApp->samples_count);
		value = 10000.0f * pApp->runtime.soundplay.Sounds[0].offset / pApp->sounddata.size;
		PX_Object_SliderBarSetValue(pApp->sliderbar_progress, (px_int)value);
		pApp->button_play->Visible = PX_FALSE;
		pApp->button_stop->Visible = PX_TRUE;
	}
}

px_void PX_ApplicationRender(PX_Application *pApp,px_dword elapsed)
{
	px_surface *pRenderSurface=&pApp->runtime.RenderSurface;
	pApp->sliderbar_progress->x = 0;
	pApp->sliderbar_progress->y = pApp->runtime.surface_height - pApp->sliderbar_progress->Height - 1;
	pApp->sliderbar_progress->Width = pApp->runtime.surface_width*1.0f;

	pApp->sliderbar_hzrange->x = pApp->runtime.surface_width - 24.f;
	pApp->sliderbar_hzrange->y = pApp->image_logo->Height;
	pApp->sliderbar_hzrange->Height = pApp->runtime.surface_height - pApp->image_logo->Height - 25;

	PX_RuntimeRenderClear(&pApp->runtime,PX_COLOR_WHITE);
	if (pApp->state==PX_APPLICATION_STATE_SHOW|| pApp->state == PX_APPLICATION_STATE_PLAYING)
	{
		if (pApp->surface_freqdisplay.width != pApp->runtime.surface_width-24 || pApp->surface_freqdisplay.height != pApp->runtime.surface_height - pApp->image_logo->Height - 25)
		{
			PX_SurfaceFree(&pApp->surface_freqdisplay);
			if (!PX_TextureCreate(&pApp->runtime.mp_ui, &pApp->surface_freqdisplay, (px_int)pApp->runtime.surface_width-24, (px_int)(pApp->runtime.surface_height - pApp->image_logo->Height - 25)))
				return;

			PX_Object_SliderBarSetRange(pApp->sliderbar_hzrange, 0, 800);
			PX_Object_SliderBarSetValue(pApp->sliderbar_hzrange, 800);
		}
		PX_Application_RenderDisplaySurface(pApp);
	}
	PX_TextureRender(pRenderSurface, &pApp->surface_freqdisplay, 0, (px_int)pApp->image_logo->Height, PX_ALIGN_LEFTTOP, 0);
	PX_ObjectRender(pRenderSurface, pApp->root, elapsed);
}

px_void PX_ApplicationPostEvent(PX_Application *pApp,PX_Object_Event e)
{
	PX_ApplicationEventDefault(&pApp->runtime, e);
	if (e.Event==PX_OBJECT_EVENT_DRAGFILE)
	{
#ifndef _MSC_VER
		extern px_byte* px_getfiledata();
		extern px_int px_getfilesize();
		px_int datasize;
		datasize = px_getfilesize();
		if (datasize)
		{
			PX_SoundStaticDataFree(&pApp->sounddata);
			if (!PX_SoundStaticDataCreate(&pApp->sounddata, &pApp->runtime.mp_resources, px_getfiledata(), datasize))
			{
				return;
			}
			pApp->progress_byte_offset = 0;
			pApp->samples_count = 0;
			pApp->state = PX_APPLICATION_STATE_PROCESSING;
			PX_Object_SliderBarSetValue(pApp->sliderbar_progress, 0);
		}
#endif
	}
	else
	PX_ObjectPostEvent(pApp->root, e);
}

